@@ -86,24 +86,9 @@ void Free_Edge_Length(t_edge *b)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-void Free_Edge_Labels(t_edge *b)
-{
-  int i;
-
-  if(b->labels)
-    {
-      For(i,b->n_labels-(b->n_labels%BLOCK_LABELS)+BLOCK_LABELS) Free(b->labels[i]);
-      Free(b->labels);
-      b->labels = NULL;
-    }
-}
-
-//////////////////////////////////////////////////////////////
-//////////////////////////////////////////////////////////////
-
 void Free_Edge(t_edge *b)
 {
-  Free_Edge_Labels(b);
+  Free_Label(b->label);
   Free_Edge_Core(b);
 }
 
@@ -126,6 +111,7 @@ void Free_Node(t_node *n)
   Free(n->s_ingrp);
   Free(n->s_outgrp);
   Free(n->cal);
+  Free_Label(n->label);
   
   if(n->c_seq_anc != NULL) 
     {
@@ -145,7 +131,6 @@ void Free_Node(t_node *n)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-
 void Free_Mat(matrix *mat)
 {
   int i;
@@ -360,7 +345,7 @@ void Free_Tree_Pars(t_tree *mixt_tree)
       Free(tree->step_mat);
       Free(tree->site_pars);
       
-      For(i,2*tree->n_otu-3) 
+      for(i=0;i<2*tree->n_otu-3;++i) 
         {
           Free_Edge_Pars(tree->a_edges[i]);
         }
@@ -898,7 +883,6 @@ void Free_Model(t_mod *mod)
 {
   Free_Custom_Model(mod);
   Free_Model_Complete(mod);
-  if(mod->m4mod) M4_Free_M4_Model(mod->m4mod);
   Free_Model_Basic(mod);
   /* Free(mod); */
 }
@@ -924,7 +908,6 @@ void Free(void *p)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-
 void Free_Input(option *io)
 {
   int i;
@@ -932,6 +915,7 @@ void Free_Input(option *io)
   do
     {
       RATES_Free_Rates(io->rates);
+      TIMES_Free_Times(io->times);
       MCMC_Free_MCMC(io->mcmc);
       Free(io->in_align_file);
       Free(io->in_tree_file);
@@ -1237,9 +1221,6 @@ void XML_Free_XML_Ds(t_ds *ds)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-//////////////////////////////////////////////////////////////
-//////////////////////////////////////////////////////////////
-
 void MCMC_Free_MCMC(t_mcmc *mcmc)
 {
   int i;
@@ -1267,29 +1248,34 @@ void MCMC_Free_MCMC(t_mcmc *mcmc)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-
-//////////////////////////////////////////////////////////////
-//////////////////////////////////////////////////////////////
-
-void M4_Free_M4_Model(m4 *m4mod)
+void TIMES_Free_Times(t_time *times)
 {
-  int i;
-
-  if(m4mod->o_mats)
-    {
-      for(i=0;i<m4mod->n_h;i++) Free(m4mod->o_mats[i]);
-      Free(m4mod->o_mats);
-      Free(m4mod->h_mat);
-      Free(m4mod->o_rr);
-      Free(m4mod->h_rr);
-      Free(m4mod->o_fq);
-      Free(m4mod->h_fq);
-      Free(m4mod->multipl);
-      Free(m4mod->multipl_unscaled);
-      Free(m4mod->h_fq_unscaled);
-    }
-
-  Free(m4mod);
+  Free(times->buff_t);
+  Free(times->nd_t);
+  Free(times->true_t);
+  Free(times->t_prior);
+  Free(times->t_mean);
+  Free(times->t_prior_min);
+  Free(times->t_prior_max);
+  Free(times->t_floor);
+  Free(times->t_has_prior);
+  Free(times->t_rank);
+  Free(times->mean_t);  
+  Free(times->t_prior_min_ori);
+  Free(times->t_prior_max_ori);
+  Free(times->times_partial_proba);  
+  Free(times->calib_prob);
+  Free(times->numb_calib_chosen);
+  for(int i=0;i<times->n_cal;i++) Free_Calib(times->a_cal[i]);
+  Free(times->a_cal);
+  Free(times->has_survived);
+  Free(times->n_jps);
+  Free(times->t_jps);
+  Free(times->time_slice_lims);
+  Free(times->n_time_slice_spans);
+  Free(times->curr_slice);
+  Free(times->has_survived);
+  Free(times);
 }
 
 //////////////////////////////////////////////////////////////
@@ -1299,26 +1285,13 @@ void RATES_Free_Rates(t_rate *rates)
 {
   if(rates->is_allocated == YES)
     {
-      int i;
       Free(rates->nd_r);
       Free(rates->br_r);
       Free(rates->buff_nd_r);
       Free(rates->buff_br_r);
       Free(rates->true_r);
-      Free(rates->buff_t);
-      Free(rates->nd_t);
-      Free(rates->true_t);
-      Free(rates->t_prior);
-      Free(rates->t_mean);
-      Free(rates->t_prior_min);
-      Free(rates->t_prior_max);
-      Free(rates->t_floor);
-      Free(rates->t_has_prior);
-      Free(rates->t_rank);
       Free(rates->dens);
       Free(rates->triplet);
-      Free(rates->n_jps);
-      Free(rates->t_jps);
       Free(rates->cond_var);
       Free(rates->invcov);
       Free(rates->ml_l);
@@ -1339,26 +1312,12 @@ void RATES_Free_Rates(t_rate *rates)
       Free(rates->cov_l);
       Free(rates->mean_l);
       Free(rates->mean_r);
-      Free(rates->mean_t);
       Free(rates->grad_l);
       Free(rates->reg_coeff);
       Free(rates->br_do_updt);
       Free(rates->cur_gamma_prior_mean);
       Free(rates->cur_gamma_prior_var);
       Free(rates->n_tips_below);
-      Free(rates->time_slice_lims);
-      Free(rates->n_time_slice_spans);
-      Free(rates->curr_slice);
-      Free(rates->has_survived);
-      Free(rates->survival_rank);
-      Free(rates->survival_dur);
-      Free(rates->calib_prob);
-      Free(rates->t_prior_min_ori);
-      Free(rates->t_prior_max_ori);
-      Free(rates->times_partial_proba);
-      Free(rates->numb_calib_chosen);
-      for(i=0;i<rates->n_cal;i++) Free_Calib(rates->a_cal[i]);
-      Free(rates->a_cal);
       Free(rates->model_name);
     }
   Free(rates);
@@ -1459,6 +1418,7 @@ void Free_Mmod(t_phyrex_mod *mmod)
   if(mmod == NULL) return;
   Free_Geo_Coord(mmod->lim_up);
   Free_Geo_Coord(mmod->lim_do);
+  Free(mmod->sigsq_scale);
   Free(mmod);
 }
 
@@ -1542,6 +1502,29 @@ void Free_TBE_Matrices(int n_otu,  short unsigned*** i_matrix, short unsigned***
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
+void Free_Label(t_label *lab)
+{
+  if(lab == NULL) return;
+  Free(lab->key);
+  Free(lab->val);
+  if(lab->next != NULL) Free_Label(lab->next);
+  Free(lab);
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+void Free_Contrasts(t_ctrst *ctrst)
+{
+  Free(ctrst->x);
+  Free(ctrst->tprime);
+  Free(ctrst);
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
