@@ -142,10 +142,11 @@ int main(int argc, char **argv)
 
                   Init_Model(cdata,mod,io);
 
+#ifdef M4
                   if(io->mod->use_m4mod) M4_Init_Model(mod->m4mod,cdata,mod);
+#endif
 
-
-                  //Make the initial tree
+                  // Make the initial tree
                   switch(io->in_tree)
                     {
                     case 0 : case 1 : { tree = Dist_And_BioNJ(cdata,mod,io); break; }
@@ -198,12 +199,6 @@ int main(int argc, char **argv)
 
                   Set_Both_Sides(YES,tree);
 
-                  if(mod->s_opt->random_input_tree)
-                    {
-                      PhyML_Printf("\n");
-                      Random_Tree(tree);
-                    }
-
                   if((!num_data_set) && (!num_tree) && (!num_rand_tree)) Check_Memory_Amount(tree);
 
                   if(io->cstr_tree && !Check_Topo_Constraints(tree,io->cstr_tree))
@@ -293,18 +288,19 @@ int main(int argc, char **argv)
                     {
                       best_lnL = tree->c_lnL;
                       if(most_likely_tree) Free(most_likely_tree);
-                      most_likely_tree = Write_Tree(tree,NO);
+                      most_likely_tree = Write_Tree(tree);
+
+                      time(&t_end);
                       
+                      Print_Fp_Out(io->fp_out_stats,t_beg,t_end,tree,
+                                   io,num_data_set+1,
+                                   (orig_random_input_tree == YES)?(num_rand_tree):(num_tree),
+                                   (num_rand_tree == io->mod->s_opt->n_rand_starts-1)?(YES):(NO), io->precision);
+
+                      if(tree->io->print_site_lnl) Print_Site_Lk(tree,io->fp_out_lk);
                     }
 
-                  time(&t_end);
 
-                  Print_Fp_Out(io->fp_out_stats,t_beg,t_end,tree,
-                               io,num_data_set+1,
-                               (orig_random_input_tree == YES)?(num_rand_tree):(num_tree),
-                               (num_rand_tree == io->mod->s_opt->n_rand_starts-1)?(YES):(NO), io->precision);
-
-                  if(tree->io->print_site_lnl) Print_Site_Lk(tree,io->fp_out_lk);
 
                   /* Start from BioNJ tree */
                   if((num_rand_tree == io->mod->s_opt->n_rand_starts-1) && (tree->mod->s_opt->random_input_tree))
@@ -362,7 +358,7 @@ int main(int argc, char **argv)
               Connect_CSeqs_To_Nodes(cdata,io,dum);
               Insert_Duplicates(dum);
               Free(most_likely_tree);
-              most_likely_tree = Write_Tree(dum,NO);
+              most_likely_tree = Write_Tree(dum);
               Free_Tree(dum);
               
               PhyML_Fprintf(io->fp_out_tree,"%s\n",most_likely_tree);
@@ -385,7 +381,6 @@ int main(int argc, char **argv)
   if(mod->s_opt->n_rand_starts > 1) PhyML_Printf("\n. Best log likelihood: %f\n",best_lnL);
 
   Free_Optimiz(mod->s_opt);
-  M4_Free_M4_Model(mod->m4mod);
   Free_Model_Basic(mod);
 
   if(io->fp_in_constraint_tree) fclose(io->fp_in_constraint_tree);
@@ -477,8 +472,8 @@ int main(int argc, char **argv)
   
 
   /* PhyML_Printf("%s\n%s", */
-  /*              Write_Tree(tree1,NO), */
-  /*              Write_Tree(tree2,NO)); */
+  /*              Write_Tree(tree1), */
+  /*              Write_Tree(tree2)); */
                
   
   /* Match_Nodes_In_Small_Tree(tree1,tree2); */
@@ -587,6 +582,7 @@ int main(int argc, char **argv)
 {
   option *io;
   int i;
+  int year;
   
   io = (option *)Get_Input(argc,argv);
   if(!io) return(0);
@@ -595,15 +591,41 @@ int main(int argc, char **argv)
 
   for(i=0;i<io->n_otu;i++)
     {
+      sscanf(io->data[i]->name,"%d",&year);
       PhyML_Printf("\n<clade id=\"clad%d\">",i+1);
       PhyML_Printf("\n\t<taxon value=\"%s\"/>",io->data[i]->name);
       PhyML_Printf("\n</clade>");
       PhyML_Printf("\n<calibration id=\"cal%d\">",i+1);
-      PhyML_Printf("\n\t<lower>0.0</lower>");
-      PhyML_Printf("\n\t<upper>0.0</upper>");
+      PhyML_Printf("\n\t<lower>%d</lower>",0);
+      PhyML_Printf("\n\t<upper>%d</upper>",0);
       PhyML_Printf("\n\t<appliesto clade.id=\"clad%d\"/>",i+1);
       PhyML_Printf("\n</calibration>");
     }
+
+  /* FILE *fp; */
+  /* char *name,*date; */
+  /* int i; */
+  
+  /* name = (char *)mCalloc(T_MAX_NAME,sizeof(char)); */
+  /* date = (char *)mCalloc(T_MAX_NAME,sizeof(char)); */
+
+  /* i = 0; */
+  /* fp = Openfile(argv[1],READ); */
+  /* do */
+  /*   { */
+  /*     if(fscanf(fp,"%s",name) == EOF) break; */
+  /*     if(fscanf(fp,"%s",date) == EOF) break; */
+  /*     PhyML_Printf("\n<clade id=\"clad%d\">",i+1); */
+  /*     PhyML_Printf("\n\t<taxon value=\"%s\"/>",name); */
+  /*     PhyML_Printf("\n</clade>"); */
+  /*     PhyML_Printf("\n<calibration id=\"cal%d\">",i+1); */
+  /*     PhyML_Printf("\n\t<lower>%s</lower>",date); */
+  /*     PhyML_Printf("\n\t<upper>%s</upper>",date); */
+  /*     PhyML_Printf("\n\t<appliesto clade.id=\"clad%d\"/>",i+1); */
+  /*     PhyML_Printf("\n</calibration>"); */
+  /*     ++i; */
+  /*   } */
+  /* while(1); */
   
 }
 
@@ -625,7 +647,7 @@ int main(int argc, char **argv)
   return 1;
 }
 
-#elif(PHYREX)
+#elif(defined PHYREX || PHYREXSIM)
 #include "phyrex.h"
 int main(int argc, char **argv)
 {
