@@ -150,26 +150,6 @@ void Make_All_Edges_Lk(t_node *a, t_node *d, t_tree *tree)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-void Make_New_Edge_Label(t_edge *b)
-{
-  int i;
-
-  b->labels = (char **)realloc(b->labels,(b->n_labels+BLOCK_LABELS)*sizeof(char *));
-
-  if(!b->labels)
-    {
-      PhyML_Printf("\n== Err. in file %s at line %d (function '%s') \n",__FILE__,__LINE__,__FUNCTION__);
-      Warn_And_Exit("");
-    }
-  else
-    {
-      for(i=b->n_labels;i<b->n_labels+BLOCK_LABELS;i++) b->labels[i] = (char *)mCalloc(T_MAX_LABEL,sizeof(char));
-    }
-}
-
-//////////////////////////////////////////////////////////////
-//////////////////////////////////////////////////////////////
-
 t_edge *Make_Edge_Light(t_node *a, t_node *d, int num)
 {
   t_edge *b;
@@ -866,13 +846,6 @@ t_mod *Make_Model_Basic()
 
 void Make_Model_Complete(t_mod *mod)
 {
-
-  if(mod->use_m4mod == YES)
-    {
-      M4_Make_Complete(mod->m4mod->n_h,mod->m4mod->n_o,mod->m4mod);
-      mod->ns = mod->m4mod->n_o * mod->m4mod->n_h;
-    }
-
   mod->Pij_rr = (vect_dbl *)mCalloc(1,sizeof(vect_dbl));
   Init_Vect_Dbl(0,mod->Pij_rr);
 
@@ -1081,6 +1054,7 @@ option *Make_Input()
   io->treelist                          = (t_treelist *)mCalloc(1,sizeof(t_treelist));
   io->mcmc                              = (t_mcmc *)MCMC_Make_MCMC_Struct();
   io->rates                             = (t_rate *)RATES_Make_Rate_Struct(-1);
+  io->times                             = (t_time *)TIMES_Make_Time_Struct(-1);
 
   return io;
 }
@@ -1270,6 +1244,48 @@ t_spr *Make_One_Spr(t_tree *tree)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
+t_time *TIMES_Make_Time_Struct(int n_otu)
+{
+  t_time *times;
+
+  times = (t_time *)mCalloc(1,sizeof(t_time));
+
+  if(n_otu > 0)
+    {
+      times->calib_prob           = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
+      times->t_prior_min_ori      = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
+      times->t_prior_max_ori      = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
+      times->times_partial_proba  = (phydbl *)mCalloc(n_otu*n_otu,sizeof(phydbl));
+      times->numb_calib_chosen    = (int *)mCalloc(n_otu*n_otu,sizeof(phydbl));
+      times->a_cal                = (t_cal **)mCalloc(10*n_otu,sizeof(t_cal *));
+      
+      times->nd_t                 = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
+      times->buff_t               = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
+      times->true_t               = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
+      times->t_mean               = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
+      times->t_prior              = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
+      times->t_prior_min          = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
+      times->t_prior_max          = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
+      times->t_floor              = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
+      times->t_rank               = (int *)mCalloc(2*n_otu-1,sizeof(int));
+      times->t_has_prior          = (short int *)mCalloc(2*n_otu-1,sizeof(short int));
+      
+      times->mean_t               = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
+      
+      times->n_jps                = (int    *)mCalloc(2*n_otu-1,sizeof(int));
+      times->t_jps                = (int    *)mCalloc(2*n_otu-2,sizeof(int));
+      
+      times->time_slice_lims      = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
+      times->n_time_slice_spans   = (int *)mCalloc(2*n_otu-1,sizeof(int));
+      times->curr_slice           = (int *)mCalloc(2*n_otu-1,sizeof(int));
+      times->has_survived         = (int    *)mCalloc(2*n_otu-1,sizeof(int));
+    }
+  return(times);
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
 t_rate *RATES_Make_Rate_Struct(int n_otu)
 {
   t_rate *rates;
@@ -1285,20 +1301,8 @@ t_rate *RATES_Make_Rate_Struct(int n_otu)
       rates->buff_br_r            = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
       rates->buff_nd_r            = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
       rates->true_r               = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
-      rates->nd_t                 = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
-      rates->buff_t               = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
-      rates->true_t               = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
-      rates->t_mean               = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
-      rates->t_prior              = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
-      rates->t_prior_min          = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
-      rates->t_prior_max          = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
-      rates->t_floor              = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
-      rates->t_rank               = (int *)mCalloc(2*n_otu-1,sizeof(int));
-      rates->t_has_prior          = (short int *)mCalloc(2*n_otu-1,sizeof(short int));
       rates->dens                 = (phydbl *)mCalloc(2*n_otu-2,sizeof(phydbl));
       rates->triplet              = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
-      rates->n_jps                = (int    *)mCalloc(2*n_otu-1,sizeof(int));
-      rates->t_jps                = (int    *)mCalloc(2*n_otu-2,sizeof(int));
       rates->cov_l                = (phydbl *)mCalloc((2*n_otu-2)*(2*n_otu-2),sizeof(phydbl));
       rates->invcov               = (phydbl *)mCalloc((2*n_otu-2)*(2*n_otu-2),sizeof(phydbl));
       rates->mean_l               = (phydbl *)mCalloc(2*n_otu-2,sizeof(phydbl));
@@ -1309,7 +1313,6 @@ t_rate *RATES_Make_Rate_Struct(int n_otu)
       rates->cov_r                = (phydbl *)mCalloc((2*n_otu-2)*(2*n_otu-2),sizeof(phydbl));
       rates->cond_var             = (phydbl *)mCalloc(2*n_otu-2,sizeof(phydbl));
       rates->mean_r               = (phydbl *)mCalloc(2*n_otu-2,sizeof(phydbl));
-      rates->mean_t               = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
       rates->lca                  = (t_node **)mCalloc((2*n_otu-1)*(2*n_otu-1),sizeof(t_node *));
       rates->reg_coeff            = (phydbl *)mCalloc((2*n_otu-2)*(2*n_otu-2),sizeof(phydbl));
       rates->trip_reg_coeff       = (phydbl *)mCalloc((2*n_otu-2)*(6*n_otu-9),sizeof(phydbl));
@@ -1325,18 +1328,6 @@ t_rate *RATES_Make_Rate_Struct(int n_otu)
       rates->cur_gamma_prior_mean = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
       rates->cur_gamma_prior_var  = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
       rates->n_tips_below         = (int *)mCalloc(2*n_otu-1,sizeof(int));
-      rates->time_slice_lims      = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
-      rates->n_time_slice_spans   = (int *)mCalloc(2*n_otu-1,sizeof(int));
-      rates->curr_slice           = (int *)mCalloc(2*n_otu-1,sizeof(int));
-      rates->has_survived         = (int    *)mCalloc(2*n_otu-1,sizeof(int));
-      rates->survival_rank        = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
-      rates->survival_dur         = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
-      rates->calib_prob           = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
-      rates->t_prior_min_ori      = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
-      rates->t_prior_max_ori      = (phydbl *)mCalloc(2*n_otu-1,sizeof(phydbl));
-      rates->times_partial_proba  = (phydbl *)mCalloc(n_otu*n_otu,sizeof(phydbl));
-      rates->numb_calib_chosen    = (int *)mCalloc(n_otu*n_otu,sizeof(phydbl));
-      rates->a_cal                = (t_cal **)mCalloc(10*n_otu,sizeof(t_cal *));
       rates->model_name           = (char *)mCalloc(T_MAX_NAME,sizeof(char));
     }
 
@@ -1388,7 +1379,7 @@ void Make_All_Calibration(t_tree *tree)
 
   For(i,2*tree->n_otu-1) all_cal[i] = Make_Calibration();
 
-  tree->rates->a_cal = all_cal;
+  tree->times->a_cal = all_cal;
 
 }
 
@@ -1622,12 +1613,13 @@ t_geo_coord *GEO_Make_Geo_Coord(int dim)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-t_phyrex_mod *PHYREX_Make_Migrep_Model(int dim)
+t_phyrex_mod *PHYREX_Make_Migrep_Model(int n_otu,int dim)
 {
   t_phyrex_mod *t;
   t = (t_phyrex_mod *)mCalloc(1,sizeof(t_phyrex_mod));
   t->lim_up = GEO_Make_Geo_Coord(dim);
   t->lim_do = GEO_Make_Geo_Coord(dim);
+  t->sigsq_scale = (phydbl *)mCalloc(2*n_otu-2,sizeof(phydbl));
   return(t);
 }
 
@@ -1747,3 +1739,67 @@ void Make_MutMap(t_tree *tree)
 
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
+
+t_label *Make_Label()
+{
+  t_label *lab;
+
+  lab = (t_label *)mCalloc(1,sizeof(t_label));
+  lab->key = (char *)mCalloc(T_MAX_KEY,sizeof(char));
+  lab->val = (char *)mCalloc(T_MAX_VAL,sizeof(char));
+  lab->next = NULL;
+
+  return(lab);
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+void Make_Contrasts(t_tree *tree)
+{
+  tree->ctrst = (t_ctrst *)mCalloc(1,sizeof(t_ctrst));
+  tree->ctrst->x = (phydbl *)mCalloc(2*tree->n_otu,sizeof(phydbl));
+  tree->ctrst->tprime = (phydbl *)mCalloc(2*tree->n_otu,sizeof(phydbl));
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+
+
