@@ -142,8 +142,8 @@ void MCMC_PHYREX_Indel_Disk(t_tree *tree);
 void MCMC_PHYREX_Delete_Disk(phydbl hr, int n_delete_disks, phydbl cur_lbda, phydbl cur_mu, phydbl cur_rad, t_tree *tree);
 void MCMC_PHYREX_Insert_Disk(phydbl hr, int n_insert_disks, phydbl cur_lbda, phydbl cur_mu, phydbl cur_rad, t_tree *tree);
 void MCMC_PHYREX_Indel_Hit(t_tree *tree);
-void MCMC_PHYREX_Insert_Hit(phydbl hr, int n_delete_disks, phydbl cur_rad, phydbl cur_mu, t_tree *tree);
-void MCMC_PHYREX_Delete_Hit(phydbl hr, int n_delete_disks, phydbl cur_rad, phydbl cur_mu, t_tree *tree);
+void MCMC_PHYREX_Insert_Hit(phydbl hr, int n_delete_disks, phydbl cur_lbda, phydbl cur_rad, phydbl cur_mu, t_tree *tree);
+void MCMC_PHYREX_Delete_Hit(phydbl hr, int n_delete_disks, phydbl cur_lbda, phydbl cur_rad, phydbl cur_mu, t_tree *tree);
 void MCMC_PHYREX_Simulate_Backward(t_tree *tree);
 void MCMC_Update_Mode(int move_num, t_mcmc *mcmc, t_tree *tree);
 void MCMC_PHYREX_Lineage_Traj(t_tree *tree);
@@ -176,5 +176,7 @@ void MCMC_PHYREX_Prune_Regraft_Local(t_tree *tree);
 void MCMC_PHYREX_Simulate_Backward(t_tree *tree);
 void MCMC_RR(t_tree *mixt_tree);
 void MCMC_PHYREX_Ldsk_Tip_To_Root(t_tree *tree);
+void MCMC_PHYREX_Neff(t_tree *tree);
+void MCMC_PHYREX_Sigsq_Scale(t_tree *tree);
 
 #endif
