@@ -20,12 +20,11 @@ the GNU public licence. See http://www.opensource.org for details.
 void PHYREX_XML(char *xml_filename);
 int PHYREX_Main(int argc, char **argv);
 int PHYREX_Main_Simulate(int argc, char **argv);
-t_tree *PHYREX_Simulate(int n_otu, int n_sites, phydbl w, phydbl h, phydbl  lbda, phydbl rad, phydbl mu, int r_seed);
 phydbl PHYREX_Lk(t_tree *tree);
+phydbl PHYREX_Lk_RW(t_tree *tree);
 phydbl PHYREX_Wrap_Lk(t_edge *b, t_tree *tree, supert_tree *stree);
 phydbl *PHYREX_MCMC(t_tree *tree);
 int PHYREX_Is_In_Disk(t_geo_coord *coord, t_dsk *disk, t_phyrex_mod *mmod);
-void PHYREX_New_Traj(t_dsk *start, t_dsk *end, t_tree *tree);
 void PHYREX_Remove_Disk(t_dsk *disk);
 void PHYREX_Insert_Disk(t_dsk *ins, t_tree *tree);
 t_ldsk *PHYREX_Prev_Coal_Lindisk(t_ldsk *t);
@@ -66,50 +65,36 @@ void PHYREX_Ldsk_To_Tree(t_tree *tree);
 void PHYREX_Ldsk_To_Tree_Post(t_node *a, t_ldsk *ldsk, int *available, t_tree *tree);
 phydbl PHYREX_Rnorm_Trunc(t_ldsk *ldsk, t_dsk *disk, t_phyrex_mod *mod);
 void PHYREX_Remove_Lindisk_Next(t_ldsk *ldsk, t_ldsk *rm);
-phydbl PHYREX_Simulate_Backward_Core(t_dsk *init_disk, int avoid_multiple_mergers, t_tree *tree);
 phydbl *PHYREX_Mean_Pairwise_Distance_Between_Lineage_Locations(t_tree *tree);
 phydbl PHYREX_Random_Select_Time_Between_Jumps(t_tree *tree);
-t_sarea *PHYREX_Simulate_Forward_Core(int n_sites, t_tree *tree);
 int PHYREX_Is_In_Ldscape(t_ldsk *ldsk, t_phyrex_mod *mmod);
 void PHYREX_Update_Lindisk_List_Core(t_dsk *disk, t_tree *tree);
 phydbl PHYREX_Mean_Time_Between_Events(t_tree *tree);
 void PHYREX_All_Pairs_Coal_Times_Dist(t_tree *tree);
 void PHYREX_Rand_Pairs_Coal_Times_Dist(t_tree *tree);
-phydbl PHYREX_Neighborhood_Size_Regression(t_tree *tree);
-phydbl PHYREX_Neighborhood_Size(t_tree *tree);
-phydbl PHYREX_Update_Radius(t_tree *tree);
-phydbl PHYREX_Update_Sigsq(t_tree *tree);
 void PHYREX_Read_Tip_Coordinates(t_tree *tree);
-phydbl PHYREX_Sample_Rad_From_Prior(t_tree *tree);
 void MCMC_PHYREX_Sigsq(t_tree *tree);
 phydbl PHYREX_LnPrior_Sigsq(t_tree *tree);
-phydbl PHYREX_Rate_Per_Unit_Area(t_tree *tree);
 phydbl PHYREX_Tree_Height(t_tree *tree);
 int PHYREX_Random_Insert_Ldsk_In_Next_List(t_ldsk *ins, t_ldsk *where);
 void PHYREX_Insert_Ldsk_In_Next_List(t_ldsk *ins, int pos, t_ldsk *where);
 t_ldsk *PHYREX_Remove_Path(t_ldsk *beg, t_ldsk *end, int *pos_end, t_tree *tree);
 void PHYREX_Insert_Path(t_ldsk *beg, t_ldsk *end, t_ldsk *path, int pos, t_tree *tree);
-t_ldsk *PHYREX_Generate_Path(t_ldsk *beg, t_ldsk *end, phydbl n_evt, phydbl sd, t_tree *tree);
-phydbl PHYREX_Path_Logdensity(t_ldsk *beg, t_ldsk *end, phydbl sd, t_tree *tree);
 phydbl PHYREX_Time_Tree_Length(t_tree *tree);
 void PHYREX_Time_Tree_Length_Pre(t_ldsk *a, t_ldsk *d, phydbl *len, t_tree *tree);
 int PHYREX_Is_On_Path(t_ldsk *target, t_ldsk *beg, t_ldsk *end);
 int PHYREX_Path_Len(t_ldsk *beg, t_ldsk *end);
 phydbl PHYREX_Lk_Core(t_dsk *disk, t_tree *tree);
+phydbl PHYREX_Lk_RW_Core(t_dsk *disk, t_tree *tree);
 void PHYREX_Print_Disk_Lk(t_tree *tree);
-phydbl PHYREX_Lk_Core_Bis(t_dsk *disk, t_tree *tree);
 t_ldsk *PHYREX_Find_Lca_Pair_Of_Ldsk(t_ldsk *n1, t_ldsk *n2, t_tree *tree);
 void PHYREX_Get_List_Of_Ancestors(t_ldsk *start, t_ldsk ***list, int *len, t_tree *tree);
 phydbl PHYREX_Dist_To_Lca(t_ldsk *d, t_ldsk *lca);
 phydbl PHYREX_Dist_Between_Two_Ldsk(t_ldsk *n1,  t_ldsk *n2, t_tree *tree);
 phydbl PHYREX_Lk_Range(t_dsk *young, t_dsk *old, t_tree *tree);
+phydbl PHYREX_Lk_RW_Range(t_dsk *young, t_dsk *old, t_tree *tree);
 void PHYREX_Print_MultiTypeTree_Config_File(int n_sites, char *filename, t_tree *tree);
-phydbl PHYREX_Effective_Density(t_tree *tree);
-phydbl PHYREX_Generation_Length(t_tree *tree);
 int PHYREX_Number_Of_Sampled_Demes(t_tree *tree);
-phydbl PHYREX_Coalescence_Rate(t_tree *tree);
-t_tree *PHYREX_Simulate_Independent_Loci(int n_otu, int n_loci, phydbl w, phydbl h, int r_seed);
-phydbl Prob_Two_Lineages_Coal_One_Event(phydbl w, phydbl h, phydbl mu, phydbl rad);
 void PHYREX_Tree_To_Ldsk(t_tree *tree);
 void PHYREX_Tree_To_Ldsk_Post(t_node *a, t_node *d, t_dsk *a_disk, t_tree *tree);
 void PHYREX_Make_And_Connect_Tip_Disks(t_tree *tree);
@@ -118,7 +103,6 @@ int PHYREX_Number_Of_Outgoing_Ldsks(t_dsk *disk);
 t_ldsk *PHYREX_Random_Select_Outgoing_Ldsk(t_dsk *disk);
 void PHYREX_Strip_And_Reconnect_Tree(t_tree *tree);
 int PHYREX_Scale_All(phydbl scale, t_dsk *start_disk, t_tree *tree);
-void PHYREX_Sample_Path(t_ldsk *young, t_ldsk *old, phydbl sd, phydbl *global_hr, t_tree *tree);
 void  PHYREX_Update_Lindisk_List_Range(t_dsk *young, t_dsk *old, t_tree *tree);
 phydbl PHYREX_Lk_Core_Range(t_dsk *young, t_dsk *old, t_tree *tree);
 int PHYREX_Number_Of_Intervals_Range(t_dsk *young, t_dsk *old, t_tree *tree);
@@ -130,6 +114,19 @@ phydbl PHYREX_Lk_Time_Component(t_tree *tree);
 phydbl PHYREX_Lk_Space_Component(t_tree *tree);
 t_ldsk *PHYREX_Find_Ldsk_From_Id(char *id, t_ldsk *root);
 t_geo_coord *PHYREX_Mean_Next_Loc(t_ldsk *ldsk, t_tree *tree);
-phydbl PHYREX_Realized_Sigsq(t_tree *tree);
+phydbl PHYREX_Root_To_Tip_Realized_Sigsq(t_tree *tree);
+phydbl PHYREX_Tip_To_Root_Realized_Sigsq(t_tree *tree);
+phydbl PHYREX_Realized_Dispersal_Dist(t_tree *tree);
+phydbl PHYREX_Tip_To_Root_Realized_Bis_Sigsq(t_tree *tree);
+void PHYREX_Label_Nodes_With_Locations(t_tree *tree);
+void PHYREX_Label_Edges(t_tree *tree);
+phydbl PHYREX_Tip_To_Root_Realized_Ter_Sigsq(t_tree *tree);
+void PHYREX_Remove_All_Disks_Except_Coal_And_Tips(t_tree *tree);
+phydbl PHYREX_Path_Logdensity(t_ldsk *young, t_ldsk *old, phydbl sd, t_tree *tree);
+void PHYREX_Sample_Path(t_ldsk *young, t_ldsk *old, phydbl sd, phydbl *global_hr, t_tree *tree);
+t_ldsk *PHYREX_Generate_Path(t_ldsk *young, t_ldsk *old, phydbl n_evt, phydbl sd, t_tree *tree);
+void PHYREX_Simulate_Backward_Core(t_dsk *disk,int avoid_multiple_mergers, t_tree *tree);
+t_tree *PHYREX_Simulate(int n_otu, int n_sites, phydbl w, phydbl h, phydbl  lbda, phydbl rad, phydbl mu, int r_seed, int modid);
+phydbl PHYREX_Update_Sigsq(t_tree *tree);
 
 #endif
