@@ -15,6 +15,8 @@ the GNU public licence. See http://www.opensource.org for details.
 #ifndef UTILITIES_H
 #define UTILITIES_H
 
+#define _POSIX_C_SOURCE 200112L
+
 
 #include <stdio.h>
 #include <stdarg.h>
@@ -92,9 +94,10 @@ static inline int isinf_d  (double      x) { return isnan (x - x); }
 static inline int isinf_ld (long double x) { return isnan (x - x); }
 #endif
 
-
-int CALL;
-int TIME;
+#define SLFV_GAUSSIAN 0 /* Spatial Lambda-Fleming-Viot model (Gaussian) */
+#define SLFV_UNIFORM 1 /* Spatial Lambda-Fleming-Viot model (Uniform) */
+#define RW  2 /* standard Brownian diffusion model in phylogeography */
+#define RRW  3 /* Lemey's relaxed random walk */
 
 #define AC 0
 #define AG 1
@@ -180,6 +183,9 @@ int TIME;
 #define T_MAX_TOKEN     200
 #define T_MAX_ID_COORD  10
 #define T_MAX_ID_DISK   10
+#define T_MAX_KEY 200
+#define T_MAX_VAL 1000
+
 
 #define N_MAX_MIXT_CLASSES 1000
 
@@ -258,14 +264,11 @@ int TIME;
 
 #define  T_MAX_FILE           200
 #define  T_MAX_LINE       2000000
-#define  T_MAX_NAME           100
+#define  T_MAX_NAME          1000
 #define  T_MAX_ID              20
 #define  T_MAX_SEQ        2000000
 #define  T_MAX_OPTION         100
-#define  T_MAX_LABEL           10
 #define  T_MAX_STATE            5
-#define  N_MAX_LABEL           10
-#define  BLOCK_LABELS         100
 
 #define  NODE_DEG_MAX        2000
 #define  BRENT_IT_MAX        1000
@@ -334,6 +337,8 @@ int TIME;
 // Amino acid ordering:
 // Ala Arg Asn Asp Cys Gln Glu Gly His Ile Leu Lys Met Phe Pro Ser Thr Trp Tyr Val
 
+#define COALESCENT 0
+
 #define COMPOUND_COR   0
 #define COMPOUND_NOCOR 1
 #define EXPONENTIAL    2
@@ -535,12 +540,13 @@ typedef struct __Node {
   struct __Lindisk_Node             *ldsk; /*! Used in PhyREX. Lineage/Disk this node corresponds to */
   struct __Node                  *rk_next; /*! Next node in the list of ranked nodes (from oldest to youngest) */
   struct __Node                  *rk_prev; /*! Previous node in the list of ranked nodes (from oldest to youngest) */
+  struct __Label                   *label;
 
 
   int                           *bip_size; /*! Size of each of the three lists from bip_node */
   int                                 num; /*! t_node number */
   int                                 tax; /*! tax = 1 -> external node, else -> internal t_node */
-  int                        check_branch; /*! check_branch=1 is the corresponding branch is labelled with '*' */
+
   char                              *name; /*! taxon name (if exists) */
   char                          *ori_name; /*! taxon name (if exists) */
   int                               n_cal; /*! Number of calibration constraints */
@@ -595,7 +601,8 @@ typedef struct __Edge {
   struct __Edge                     *prev;
   struct __Edge                *next_mixt;
   struct __Edge                *prev_mixt;
-
+  struct __Label                   *label;
+  
   int                                 num; /*! branch number */
   scalar_dbl                           *l; /*! branch length */
   scalar_dbl                      *best_l; /*! best branch length found so far */
@@ -680,8 +687,6 @@ typedef struct __Edge {
   phydbl                      support_val;
 
 
-  char                           **labels; /*! string of characters that labels the corresponding t_edge */
-  int                            n_labels; /*! number of labels */
   int                             n_jumps; /*! number of jumps of substitution rates */
 
   int                    *n_diff_states_l; /*! Number of different states found in the subtree on the left of this edge */
@@ -692,6 +697,8 @@ typedef struct __Edge {
   short int        update_partial_lk_left;
   short int        update_partial_lk_rght;
 
+  
+  
 }t_edge;
 
 /*!********************************************************/
@@ -717,7 +724,8 @@ typedef struct __Tree{
   struct __SPR            **spr_list_all_edge;
   struct __SPR                      *best_spr;
   struct __Tdraw                     *ps_tree; /*! structure for drawing trees in postscript format */
-  struct __T_Rate                       *rates; /*! structure for handling rates of evolution */
+  struct __T_Rate                      *rates; /*! structure for handling rates of evolution */
+  struct __T_Time                      *times; /*! structure for handling node ages */
   struct __Tmcmc                        *mcmc;
   struct __Phylogeo                      *geo;
   struct __Migrep_Model                 *mmod;
@@ -726,7 +734,8 @@ typedef struct __Tree{
   struct __XML_node                 *xml_root;
   struct __Generic_LL              *edge_list;
   struct __Generic_LL              *node_list;
-
+  struct __Independent_Contrasts       *ctrst; /*! Pointer to data structure used for independent contrasts */
+  
 
   short int                         eval_alnL; /*! Evaluate likelihood for genetic data */
   short int                         eval_rlnL; /*! Evaluate likelihood for rates along the tree */
@@ -1181,6 +1190,7 @@ typedef struct __Option { /*! mostly used in 'help.c' */
   struct __Option              *prev;
   struct __Tmcmc               *mcmc;
   struct __T_Rate             *rates;
+  struct __T_Time             *times;
   
   int                    interleaved; /*! interleaved or sequential sequence file format ? */
   int                        in_tree; /*! =1 iff a user input tree is used as input */
@@ -1316,23 +1326,24 @@ typedef struct __Option { /*! mostly used in 'help.c' */
 /*!********************************************************/
 
 typedef struct __Optimiz { /*! parameters to be optimised (mostly used in 'optimiz.c') */
-  int                opt_alpha; /*! =1 -> the gamma shape parameter is optimised */
-  int                opt_kappa; /*! =1 -> the ts/tv ratio parameter is optimised */
-  int               opt_lambda; /*! =1 -> the F84|TN93 model specific parameter is optimised */
-  int               opt_pinvar; /*! =1 -> the proportion of invariants is optimised */
-  int           opt_state_freq; /*! =1 -> the nucleotide frequencies are optimised */
-  int                   opt_rr; /*! =1 -> the relative rate parameters of the GTR or the customn model are optimised */
-  int          opt_subst_param; /*! if opt_topo=0 and opt_subst_param=1 -> the numerical parameters of the
+  short int                opt_alpha; /*! =1 -> the gamma shape parameter is optimised */
+  short int                opt_kappa; /*! =1 -> the ts/tv ratio parameter is optimised */
+  short int               opt_lambda; /*! =1 -> the F84|TN93 model specific parameter is optimised */
+  short int               opt_pinvar; /*! =1 -> the proportion of invariants is optimised */
+  short int           opt_state_freq; /*! =1 -> the nucleotide frequencies are optimised */
+  short int                   opt_rr; /*! =1 -> the relative rate parameters of the GTR or the customn model are optimised */
+  short int          opt_subst_param; /*! if opt_topo=0 and opt_subst_param=1 -> the numerical parameters of the
                    model are optimised. if opt_topo=0 and opt_free_param=0 -> no parameter is
                    optimised */
-  int            opt_cov_delta;
-  int            opt_cov_alpha;
-  int       opt_cov_free_rates;
+  short int            opt_cov_delta;
+  short int            opt_cov_alpha;
+  short int       opt_cov_free_rates;
+  
+  short int              opt_clock_r;
 
-
-  int                   opt_bl; /*! =1 -> the branch lengths are optimised */
-  int                 opt_topo; /*! =1 -> the tree topology is optimised */
-  int              topo_search;
+  short int                   opt_bl; /*! =1 -> the branch lengths are optimised */
+  short int                 opt_topo; /*! =1 -> the tree topology is optimised */
+  short int              topo_search;
   phydbl               init_lk; /*! initial loglikelihood value */
   int                 n_it_max; /*! maximum bnumber of iteration during an optimisation step */
   int                 last_opt; /*! =1 -> the numerical parameters are optimised further while the
@@ -1510,21 +1521,9 @@ typedef struct __T_Rate {
   phydbl lexp; /*! Parameter of the exponential distribution that governs the rate at which substitution between rate classes ocur */
   phydbl alpha;
   phydbl less_likely;
-  phydbl birth_rate;
-  phydbl birth_rate_min;
-  phydbl birth_rate_max;
-  phydbl birth_rate_pivot;
-  phydbl death_rate;
-  phydbl death_rate_min;
-  phydbl death_rate_max;
-  phydbl death_rate_pivot;
-  phydbl min_rate;
-  phydbl max_rate;
   phydbl c_lnL1;
   phydbl c_lnL2;
   phydbl c_lnL_rates; /*! Prob(Br len | time stamps, model of rate evolution) */
-  phydbl c_lnL_times; /*! Prob(time stamps) */
-  phydbl c_lnL_jps; /*! Prob(# Jumps | time stamps, rates, model of rate evolution) */
   phydbl clock_r; /*! Mean substitution rate, i.e., 'molecular clock' rate */
   phydbl min_clock;
   phydbl max_clock;
@@ -1540,12 +1539,12 @@ typedef struct __T_Rate {
   phydbl covdet;
   phydbl sum_invalid_areas;
 
+  phydbl min_rate;
+  phydbl max_rate;
 
   phydbl     *nd_r;  /*! Current rates at nodes */
   phydbl     *br_r;  /*! Current rates along edges */
-  phydbl     *nd_t; /*! Current t_node times */
   phydbl     *triplet;
-  phydbl     *true_t; /*! true t_node times (including root node) */
   phydbl     *true_r; /*! true t_edge rates (on rooted tree) */
   phydbl     *buff_t;
   phydbl     *buff_br_r;
@@ -1558,7 +1557,6 @@ typedef struct __T_Rate {
   phydbl     *invcov;
   phydbl     *cov_r;
   phydbl     *mean_r; /*! average values of br_r taken across the sampled values during the MCMC */
-  phydbl     *mean_t; /*! average values of nd_t taken across the sampled values during the MCMC */
   phydbl     *_2n_vect1;
   phydbl     *_2n_vect2;
   phydbl     *_2n_vect3;
@@ -1570,20 +1568,10 @@ typedef struct __T_Rate {
   phydbl     *trip_reg_coeff;
   phydbl     *cond_var;
   phydbl     *reg_coeff;
-  phydbl     *t_prior;
-  phydbl     *t_prior_min;
-  phydbl     *t_prior_max;
-  phydbl     *t_floor;
-  phydbl     *t_mean;
-  int        *t_rank; /* rank of nodes, e.g., tree->nd_a[tree->rates->t_rank[0]] is the oldest node */
   phydbl     *mean_l;
   phydbl     *cov_l;
   phydbl     *grad_l; /* gradient */
   phydbl     inflate_var;
-  phydbl     *time_slice_lims;
-  phydbl     *survival_rank;
-  phydbl     *survival_dur;
-  phydbl     *calib_prob;
 
   int adjust_rates; /*! if = 1, branch rates are adjusted such that a modification of a given t_node time
                does not modify any branch lengths */
@@ -1598,46 +1586,89 @@ typedef struct __T_Rate {
   int update_mean_l;
   int update_cov_l;
 
-  int *n_jps;
-  int *t_jps;
-  int n_time_slices;
-  int *n_time_slice_spans;
-  int *curr_slice;
   int *n_tips_below;
 
-  short int *t_has_prior;
   struct __Node **lca; /*! 2-way table of common ancestral nodes for each pair of nodes */
 
   short int *br_do_updt;
   phydbl *cur_gamma_prior_mean;
   phydbl *cur_gamma_prior_var;
 
-  short int nd_t_recorded;
   short int br_r_recorded;
 
-  int *has_survived;
-
-  struct __Calibration **a_cal; /* array of calibration data */
-  int                    n_cal; /* number of elements in a_cal */
-
-  phydbl     *t_prior_min_ori;
-  phydbl     *t_prior_max_ori;
-  phydbl     *times_partial_proba;
-
   phydbl log_K_cur;
   int cur_comb_numb;
-  int *numb_calib_chosen;
-  int update_time_norm_const;
 
   struct __T_Rate *next;
   struct __T_Rate *prev;
 
-  short int is_asynchronous;
-  short int clock_r_fixed;
 }t_rate;
 
 /*!********************************************************/
 
+typedef struct __T_Time {
+  phydbl *nd_t; /*! Current t_node times */
+  phydbl *buff_t; /*! Current t_node times */
+  phydbl *true_t; /*! Current t_node times */
+
+  phydbl c_lnL_times; /*! Prob(time stamps) */
+  phydbl c_lnL_jps; /*! Prob(# Jumps | time stamps, rates, model of rate evolution) */
+
+  phydbl scaled_pop_size; // Product of effective population size with length of a generation in calendar time unit
+
+  phydbl birth_rate;
+  phydbl birth_rate_min;
+  phydbl birth_rate_max;
+  phydbl birth_rate_pivot;
+
+  phydbl death_rate;
+  phydbl death_rate_min;
+  phydbl death_rate_max;
+  phydbl death_rate_pivot;
+
+  phydbl *t_prior;
+  phydbl *t_prior_min;
+  phydbl *t_prior_max;
+  phydbl *t_floor;
+  phydbl *t_mean;
+  int    *t_rank; /* rank of nodes, e.g., tree->nd_a[tree->rates->t_rank[0]] is the oldest node */
+  
+  short int nd_t_recorded;
+
+  short int is_asynchronous;
+
+  phydbl *time_slice_lims;
+  phydbl *calib_prob;
+
+  struct __Calibration **a_cal; /* array of calibration data */
+  int n_cal; /* number of elements in a_cal */
+
+  phydbl *t_prior_min_ori;
+  phydbl *t_prior_max_ori;
+  phydbl *times_partial_proba;
+
+  int *has_survived;
+
+  int *n_jps;
+  int *t_jps;
+
+  int *numb_calib_chosen;
+
+  int n_time_slices;
+  int *n_time_slice_spans;
+  int *curr_slice;
+
+  short int *t_has_prior;
+
+  phydbl *mean_t; /*! average values of nd_t taken across the sampled values during the MCMC */
+
+  int model;
+
+  int update_time_norm_const;
+}t_time;
+
+/*!********************************************************/
+
 typedef struct __Tmcmc {
   struct __Option *io;
 
@@ -1701,6 +1732,7 @@ typedef struct __Tmcmc {
   int num_move_phyrex_scale_times;
   int num_move_phyrex_ldscape_lim;
   int num_move_phyrex_sigsq;
+  int num_move_phyrex_neff;
   int num_move_phyrex_sim;
   int num_move_phyrex_traj;
   int num_move_phyrex_indel_disk_serial;
@@ -1714,6 +1746,7 @@ typedef struct __Tmcmc {
   int num_move_phyrex_add_remove_jump;
   int num_move_clade_change;
   int num_move_phyrex_ldsk_tip_to_root;
+  int num_move_phyrex_sigsq_scale;
 
   int nd_t_digits;
   int *monitor;
@@ -1899,15 +1932,24 @@ typedef struct __Phylogeo{
   phydbl                dum; // dummy parameter use to assess non-identifiability issues
   phydbl            min_dum;
   phydbl            max_dum;
-
   
 }t_geo;
 
 /*!********************************************************/
 // Structure for the Etheridge-Barton migration/reproduction model
 typedef struct __Migrep_Model{
-  int                           name;
+  struct __Geo_Coord         *lim_up; // max longitude and lattitude                        
+  struct __Geo_Coord         *lim_do; // min longitude and lattitude                        
+  phydbl                *sigsq_scale; // Scaling factors for the variance parameter in RRW model
+
+  phydbl             sigsq_scale_min;
+  phydbl             sigsq_scale_max;
+  
+  short int                       id;
   int                          n_dim;
+  int                    safe_phyrex;
+  int           max_num_of_intervals;
+  int                     update_rad;
 
   phydbl                        lbda; // rate at which events occur
   phydbl                    min_lbda; // min of rate at which events occur
@@ -1923,7 +1965,6 @@ typedef struct __Migrep_Model{
   phydbl                     min_rad; // min of radius of the migrep disk 
   phydbl                     max_rad; // max of radius of the migrep disk 
   phydbl             prior_param_rad; // parameter of the parameter for the prior on radius
-  int                     update_rad;
 
   phydbl                       sigsq; // parent to offspring distance variance (i.e., gene flow) parameter. 
   phydbl                   min_sigsq; // min 
@@ -1932,7 +1973,7 @@ typedef struct __Migrep_Model{
 
   phydbl                         rho; // intensity parameter of the Poisson point processs
   phydbl                gen_cal_time; // duration of one generation in calendar time unit
-
+  phydbl                          nu; // parameter of hyperprior on sigsq_scale (see Eq. (1) in Lemey et al., 2010).
   
   phydbl                       c_lnL; // current value of log-likelihood 
   phydbl              c_ln_prior_rad; // current value of log prior for the prior on radius
@@ -1940,13 +1981,9 @@ typedef struct __Migrep_Model{
   phydbl               c_ln_prior_mu; // current value of log prior for the prior on mu
   phydbl            c_ln_prior_sigsq; // current value of log prior for the prior on sigsq=4.pi.lbda.mu.rad^4
 
-  int                    safe_phyrex;
   phydbl             soft_bound_area;
 
-  struct __Geo_Coord            *lim_up; // max longitude and lattitude                        
-  struct __Geo_Coord            *lim_do; // min longitude and lattitude                        
 
-  struct __SampArea       *samp_area;
 }t_phyrex_mod;
 
 /*!********************************************************/
@@ -1968,6 +2005,15 @@ typedef struct __Disk_Event{
 
 /*!********************************************************/
 
+// Data structure for implementing Felsenstein's independent contrasts likelihood calculation
+// Use the same notation as that in the 1973 article (Am J Hum Genet 25:471-492)
+typedef struct __Independent_Contrasts{
+  phydbl *x;
+  phydbl *tprime;
+}t_ctrst;
+
+/*!********************************************************/
+
 typedef struct __Geo_Coord{
   phydbl             *lonlat; /* longitude-latitude vector */
   int                    dim;
@@ -2028,6 +2074,18 @@ typedef struct __JSON_Array {
 }json_a;
 
 /*!********************************************************/
+
+
+typedef struct __Label{
+  char *key;
+  char *val;
+  char sep;
+  struct __Label    *next;
+}t_label;
+
+
+
+
 /*!********************************************************/
 /*!********************************************************/
 /*!********************************************************/
@@ -2326,6 +2384,9 @@ t_tree *Duplicate_Tree(t_tree *ori);
 matrix *K80_dist(calign *data, phydbl g_shape);
 matrix *JC69_Dist(calign *data, t_mod *mod);
 matrix *Hamming_Dist(calign *data, t_mod *mod);
+phydbl Haversine_Distance(phydbl lon1, phydbl lat1, phydbl lon2, phydbl lat2);
+phydbl Tree_Length(t_tree *tree);
+void Remove_Duplicates_From_Tree(calign *data, t_tree *tree);
 
 
 #include "xml.h"
@@ -2354,6 +2415,9 @@ matrix *Hamming_Dist(calign *data, t_mod *mod);
 
 #ifdef PHYREX
 #include "phyrex.h"
+#include "rw.h"
+#include "rrw.h"
+#include "slfv.h"
 #endif
 
 #ifdef MPI
