@@ -173,6 +173,7 @@ t_tree *XML_Process_Base(char *xml_filename)
       io->fp_out_stats = Openfile(io->out_stats_file,WRITE);
     }
 
+               
   s = XML_Get_Attribute_Value(p_elem,"print.trace");
 
   if(s)
@@ -211,7 +212,24 @@ t_tree *XML_Process_Base(char *xml_filename)
           io->fp_out_json_trace = Openfile(io->out_json_trace_file,READWRITE);
         }
     }
+
+  s = XML_Get_Attribute_Value(p_elem,"print.site.lk");
   
+  if(s)
+    {
+      select = XML_Validate_Attr_Int(s,6,
+                                     "true","yes","y",
+                                     "false","no","n");
+      if(select < 3) io->print_site_lnl = YES;
+      
+      strcpy(io->out_lk_file,outputfile);
+      strcat(io->out_lk_file, "_phyml_lk");
+      if(io->append_run_ID) { strcat(io->out_lk_file,"_"); strcat(io->out_lk_file,io->run_id_string); }
+      strcat(io->out_lk_file, ".txt");
+      io->fp_out_lk = Openfile(io->out_lk_file,WRITE);
+    }
+
+      
   s = XML_Get_Attribute_Value(p_elem,"branch.test");
   if(s)
     {
@@ -367,23 +385,9 @@ t_tree *XML_Process_Base(char *xml_filename)
       
       /*! Open pointer to alignment
        */
-      io->fp_in_align = Openfile(io->in_align_file,0);
+      io->fp_in_align = Openfile(io->in_align_file,READ);
       
 
-      s = XML_Get_Attribute_Value(p_elem,"print.site.lk");
-      if(s)
-        {
-          select = XML_Validate_Attr_Int(s,6,
-                                         "true","yes","y",
-                                         "false","no","n");
-          if(select < 3) io->print_site_lnl = YES;
-
-          strcpy(io->out_lk_file,io->in_align_file);
-          strcat(io->out_lk_file, "_phyml_lk.txt");
-          io->fp_out_lk = Openfile(io->out_lk_file,1);
-        }
-
-
       /*! Load sequence file
        */
       io->data = Get_Seq(io);
@@ -1032,7 +1036,7 @@ t_tree *XML_Process_Base(char *xml_filename)
                             }
                           else
                             {
-                              For(i,2*tree->n_otu-1)
+                              for(i=0;i<2*tree->n_otu-1;++i)
                                 {
                                   Free_Scalar_Dbl(tree->a_edges[i]->l);
                                   Free_Scalar_Dbl(tree->a_edges[i]->l_old);
@@ -1863,10 +1867,6 @@ xml_node *XML_Search_Node_Attribute_Value(char *attr_name, char *value, int skip
   return NULL;
 }
 
-//////////////////////////////////////////////////////////////
-//////////////////////////////////////////////////////////////
-
-
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
@@ -2347,14 +2347,14 @@ void XML_Read_Calibration(xml_node *xroot, t_tree *tree)
                           
           cal->clade_list_size = 0;
           cal->current_clade_idx = 0;
-          cal->lower = -up;
-          cal->upper = -low;
+          cal->lower = low;
+          cal->upper = up;
           cal->is_primary = YES;
 
-          tree->rates->a_cal[tree->rates->n_cal] = cal;
-          tree->rates->n_cal++;
+          tree->times->a_cal[tree->times->n_cal] = cal;
+          tree->times->n_cal++;
           
-          if(tree->rates->n_cal > 10 * tree->n_otu)
+          if(tree->times->n_cal > 10 * tree->n_otu)
             {
               PhyML_Fprintf(stderr,"\n. There are too many clades defined that are not found in the tree..."); 
               PhyML_Fprintf(stderr,"\n. Please remove some of them.");
@@ -2460,9 +2460,9 @@ void XML_Read_Calibration(xml_node *xroot, t_tree *tree)
 
   
   PhyML_Printf("\n\n.......................................................................");
-  for(i=0;i<tree->rates->n_cal;++i)
+  for(i=0;i<tree->times->n_cal;++i)
     {
-      cal = tree->rates->a_cal[i];
+      cal = tree->times->a_cal[i];
 
       phydbl sum = 0.0;
       for(j=0;j<cal->clade_list_size;j++) sum += cal->alpha_proba_list[j];
@@ -2482,9 +2482,9 @@ void XML_Read_Calibration(xml_node *xroot, t_tree *tree)
 
   t_ref = t = -1.;
   
-  for(i=0;i<tree->rates->n_cal;++i)
+  for(i=0;i<tree->times->n_cal;++i)
     {
-      cal = tree->rates->a_cal[i];
+      cal = tree->times->a_cal[i];
       for(j=0;j<cal->clade_list_size;j++)
         {
           clade = cal->clade_list[j];
@@ -2502,7 +2502,7 @@ void XML_Read_Calibration(xml_node *xroot, t_tree *tree)
 
                   if(Are_Equal(t,t_ref,1.E-10) == NO)
                     {
-                      tree->rates->is_asynchronous = YES;
+                      tree->times->is_asynchronous = YES;
                       break;
                     }
                 }                    
@@ -2511,7 +2511,7 @@ void XML_Read_Calibration(xml_node *xroot, t_tree *tree)
     }
   
   PhyML_Printf("\n\n");
-  PhyML_Printf("\n. Is asynchronous: %s",tree->rates->is_asynchronous ? "yes" : "no");
+  PhyML_Printf("\n. Is asynchronous: %s",tree->times->is_asynchronous ? "yes" : "no");
 }
 
 //////////////////////////////////////////////////////////////
